Program AppleMan;


(**************************************************************************)
(*                                                                        *)
(*  AppleMan 1.0, a demo program for GNU Pascal and BGI2GRX which shows   *)
(*  a graph of the Mandelbrot set on a standard VGA display.  It can be   *)
(*  compiled both with Borland Pascal and GNU Pascal.                     *)
(*                                                                        *)
(*  Copyright (C) 1997  Peter Gerwinski, Essen, Germany                   *)
(*                      peter.gerwinski@uni-essen.de                      *)
(*                      http://home.pages.de/~peter.gerwinski/            *)
(*                                                                        *)
(*  This program is free software; you can redistribute it and/or modify  *)
(*  it under the terms of the GNU General Public License, version 2, as   *)
(*  published by the Free Software Foundation.  You can use parts of      *)
(*  this program in your own programs under the terms of the GNU Library  *)
(*  General Public License, version 2.                                    *)
(*                                                                        *)
(*  This program is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  and the GNU Library General Public License along with this program;   *)
(*  if not, write to the Free Software Foundation, Inc., 675 Mass Ave,    *)
(*  Cambridge, MA 02139, USA.                                             *)
(*                                                                        *)
(**************************************************************************)


(*$ifdef __GPC__ *)  (* GNU Pascal *)

uses
  Graph;    (* Link with -lbp -lbcc2 -lgrx20 (in this order). *)

(*$else *)  (* Borland Pascal *)

(*$N+,E+*)  (* With E+, and in protected mode, the main loop *)
            (* loses 0.1 s in the first run.                :*)

uses
  Dos, Graph;

Type
  Real = Double;

(*$endif *)


Const
  GraphPath = 'F:\BP\BGI';  (* Customize here, if necessary. *)
  tmax = 255;


(*$ifdef __GPC__ *)


Type
  TimeRec = record
    min, hour, hund, sec: __byte__ Integer;
  end (* TimeRec *);


Var
  Timer1, Timer2: Integer;


Procedure djGetTime ( Var T: TimeRec ); AsmName 'gettime';


Function GetTimer: Integer;

Var
  T: TimeRec;

begin (* GetTime *)
  djGetTime ( T );
  GetTimer:= ( ( T.hour * 60 + T.min ) * 60 + T.sec ) * 100 + T.hund;
end (* GetTime *);


(*$else *)


Var
  Timer1, Timer2: LongInt;


Function GetTimer: LongInt;

Var
  hour, min, sec, hund: Word;

begin (* GetTimer *)
  GetTime ( hour, min, sec, hund );
  GetTimer:= ( ( hour * LongInt ( 60 )
               + min ) * LongInt ( 60 )
               + sec ) * LongInt ( 100 ) + hund;
end (* GetTimer *);


(*$endif *)


Procedure Init;

Var
  GraphDriver, GraphMode: Integer;

begin (* Init *)
  GraphDriver:= Vga;
  GraphMode:= VgaHi;
  InitGraph ( GraphDriver, GraphMode, GraphPath );
  Timer1:= GetTimer;
end (* Init *);


Procedure DoAppleMan;

Var
  gx, gy, t: Integer;
  x, y, z, xx, yy: Real;

begin (* DoAppleMan *)
  gy:= 0;
  repeat
    gx:= 0;
    repeat
      t:= 0;
      xx:= gx * ( 3 / GetMaxX ) - 2;
      yy:= 1.1 - gy * ( 2.2 / GetMaxY );
      if ( sqr ( xx + 0.24 ) + sqr ( yy ) > 0.24 )
       and ( sqr ( xx + 1 ) + sqr ( yy ) > 0.05 ) then
        begin
          x:= 0;
          y:= 0;
          repeat
            z:= sqr ( x ) - sqr ( y ) + xx;
            y:= 2 * x * y + yy;
            x:= z;
            inc ( t );
          until ( abs ( z ) > 100 ) or ( t > tmax );
          if t <= tmax then
            PutPixel ( gx, gy, t );
        end (* if *);
      inc ( gx );
    until gx > GetMaxX;
    inc ( gy );
  until gy > GetMaxY;
end (* DoAppleMan *);


Procedure Fini;

begin (* Fini *)
  Timer2:= GetTimer;
  readln;
  CloseGraph;
  writeln ( 'Calculation time for this run:              ',
            ( Timer2 - Timer1 ) * 0.01 : 10 : 1, ' s' );
  writeln ( 'Times measured on Esmeralda (Pentium 166):' );
  writeln ( 'Borland Pascal, real mode:                  ',
            6.4 : 10 : 1, ' s' );
  writeln ( 'Borland Pascal, PM, FPU emulator not linked:',
            8.6 : 10 : 1, ' s' );
  writeln ( 'Borland Pascal, PM, FPU emulator linked:    ',
            8.7 : 10 : 1, ' s' );
  writeln ( 'GNU Pascal, always PM, without optimization:',
            6.7 : 10 : 1, ' s' );
  writeln ( 'GNU Pascal with optimization -O, -O2, -O3:  ',
            2.7 : 10 : 1, ' s' );
  readln;
end (* Fini *);


begin
  Init;
  DoAppleMan;
  Fini;
end.

